/*
   Author:
    rbe
    
   Description:
    RUBE library core data and functions.
*/

/**********************************************************
 * default spawn distances
 */
//  for structures
RUBE_SPAWNDISTANCE1 = (1500 max viewDistance);
// ... and for units
RUBE_SPAWNDISTANCE2 = (1005 max (viewDistance * 0.67));


/**********************************************************
 * generic all-purpose unique id
 */
RUBE_INTERN_AUTOID_FORMAT = "RUBE_AUTOID_%1";
RUBE_INTERN_AUTOID = 0;
RUBE_INTERN_AUTOMRKID_FORMAT = "RUBE_AUTOMRKID_%1"; 
RUBE_INTERN_AUTOMRKID = 0;
RUBE_createID = {
   private ["_id"];
   _id = format [RUBE_INTERN_AUTOID_FORMAT, RUBE_INTERN_AUTOID];
   RUBE_INTERN_AUTOID = RUBE_INTERN_AUTOID + 1;
   _id
};
// separate id generator and clear all function for markers,
// so we can screw around, clear everything drawn with auto-ids
// and start again... yeah!
RUBE_createMarkerID = {
   private ["_id"];
   _id = format [RUBE_INTERN_AUTOMRKID_FORMAT, RUBE_INTERN_AUTOMRKID];
   RUBE_INTERN_AUTOMRKID = RUBE_INTERN_AUTOMRKID + 1;
   _id
};
RUBE_clearAutoMarkers = {
   for "_i" from 0 to (RUBE_INTERN_AUTOMRKID - 1) do
   {
      deleteMarker format[RUBE_INTERN_AUTOMRKID_FORMAT, _i];
   };
   RUBE_INTERN_AUTOMRKID = 0;
   true
};


/**********************************************************
 * generic memory/data to avoid globals
 */
// Nothing can be really removed from this list to guarantee 
// data integrity. However we may "nullify" unneeded data.
RUBE_MEMORY_DATA = [];
// any => index
RUBE_saveData = {
   private ["_index"];
   _index = count RUBE_MEMORY_DATA;
   RUBE_MEMORY_DATA set [_index, _this];
   _index
};
// index => any
RUBE_getData = {
   if (_this < (count RUBE_MEMORY_DATA)) exitWith
   {
      (RUBE_MEMORY_DATA select _this)
   };
   objNull
};
// [index, any] => boolean
RUBE_setData = {
   if ((_this select 0) < (count RUBE_MEMORY_DATA)) exitWith
   {
      RUBE_MEMORY_DATA set [(_this select 0), (_this select 1)];
      true
   };
   false
};


/**********************************************************
 * generic, little, ...
 */

// to easily check boolean variables in some objects scope
// (getVariable wrapper function)
// [obj, var] => boolean
RUBE_isTrue = {
   private ["_var"];
   _var = (_this select 0) getVariable (_this select 1);
   
   if (isNil ("_var")) exitWith 
   { 
      false 
   };   
   
   if ((typeName _var) != "BOOL") exitWith
   {
      false
   };
   
   _var
};

// retrieves a variable from an objects scope or returns a default
//  default defaultValue is false
// (getVariable wrapper function)
// [obj, var, (defaultValue)] => boolean
RUBE_initVariable = {
   private ["_var", "_default"];
   _var = (_this select 0) getVariable (_this select 1);
   _default = false;
   if ((count _this) > 2) then
   {
      _default = _this select 2;
   };
   
   if (isNil ("_var")) exitWith 
   { 
      _default 
   };  
   
   _var
};


/*
----------------------------------------------------------------------------
  DEPRECATED(!) since OA 1.55 and the introduction of getTerrainHeightASL
----------------------------------------------------------------------------
*/
// probe object (to setPos and getPosASL)
//RUBE_PROBE_OBJ = createVehicle ["Baseball", [0,0,0], [], 0, "NONE"];



/**********************************************************
 * debug functions
 */

// outputs the given information via diag_log into the rpt-file
//  only if the variable RUBE_DEBUG is defined in the callers scope 
//  and evaluates to true. 
// string => void (rpt output)
RUBE_debugLog = {
   if (isNil("RUBE_DEBUG")) exitWith {};
   if (RUBE_DEBUG) then
   {
      diag_log _this;
   };
};

// hint's only if RUBE_DEBUG is defined and true.
// string => void (default hint)
RUBE_debugHint = {
   if (isNil("RUBE_DEBUG")) exitWith {};
   if (RUBE_DEBUG) then
   {
      hint _this;
   };
};

// wrapper for debug markers
RUBE_debugDrawMarker = {
   if (isNil("RUBE_DEBUG")) exitWith {};
   if (RUBE_DEBUG) then
   {
      _this call RUBE_mapDrawMarker;
   };
};

// wrapper for debug lines
RUBE_debugDrawLine = {
   if (isNil("RUBE_DEBUG")) exitWith {};
   if (RUBE_DEBUG) then
   {
      _this call RUBE_mapDrawLine;
   };
};